@@ -19,11 +19,37 @@ def _compare_tables(conn_table_names, metadata_table_names,
                     object_filters,
                     inspector, metadata, diffs, autogen_context):
 
+    default_schema = inspector.bind.dialect.default_schema_name
+
+    # tables coming from the connection will not have "schema"
+    # set if it matches default_schema_name; so we need a list
+    # of table names from local metadata that also have "None" if schema
+    # == default_schema_name.  Most setups will be like this anyway but
+    # some are not (see #170)
+    metadata_table_names_no_dflt_schema = OrderedSet([
+        (schema if schema != default_schema else None, tname)
+        for schema, tname in metadata_table_names
+    ])
+
+    # to adjust for the MetaData collection storing the tables either
+    # as "schemaname.tablename" or just "tablename", create a new lookup
+    # which will match the "non-default-schema" keys to the Table object.
+    tname_to_table = dict(
+                        (
+                            no_dflt_schema,
+                            metadata.tables[sa_schema._get_table_key(tname, schema)]
+                        )
+                        for no_dflt_schema, (schema, tname) in zip(
+                                    metadata_table_names_no_dflt_schema,
+                                    metadata_table_names)
+                        )
+    metadata_table_names = metadata_table_names_no_dflt_schema
+
     for s, tname in metadata_table_names.difference(conn_table_names):
         name = '%s.%s' % (s, tname) if s else tname
-        metadata_table = metadata.tables[sa_schema._get_table_key(tname, s)]
+        metadata_table = tname_to_table[(s, tname)]
         if _run_filters(metadata_table, tname, "table", False, None, object_filters):
-            diffs.append(("add_table", metadata.tables[name]))
+            diffs.append(("add_table", metadata_table))
             log.info("Detected added table %r", name)
             _compare_indexes_and_uniques(s, tname, object_filters,
                     None,
@@ -55,7 +81,7 @@ def _compare_tables(conn_table_names, metadata_table_names,
 
     for s, tname in sorted(existing_tables):
         name = '%s.%s' % (s, tname) if s else tname
-        metadata_table = metadata.tables[name]
+        metadata_table = tname_to_table[(s, tname)]
         conn_table = existing_metadata.tables[name]
 
         if _run_filters(metadata_table, tname, "table", False, conn_table, object_filters):
@@ -101,16 +127,10 @@ def _compare_columns(schema, tname, object_filters, conn_table, metadata_table,
             log.info("Detected added column '%s.%s'", name, cname)
 
     for cname in set(conn_col_names).difference(metadata_col_names):
-        rem_col = sa_schema.Column(
-                    cname,
-                    conn_table.c[cname].type,
-                    nullable=conn_table.c[cname].nullable,
-                    server_default=conn_table.c[cname].server_default
-                )
-        if _run_filters(rem_col, cname,
+        if _run_filters(conn_table.c[cname], cname,
                                 "column", True, None, object_filters):
             diffs.append(
-                ("remove_column", schema, tname, rem_col)
+                ("remove_column", schema, tname, conn_table.c[cname])
             )
             log.info("Detected removed column '%s.%s'", name, cname)
 
@@ -177,9 +197,9 @@ class _ix_constraint_sig(_constraint_sig):
 
 def _get_index_column_names(idx):
     if compat.sqla_08:
-        return [exp.name for exp in idx.expressions]
+        return [getattr(exp, "name", None) for exp in idx.expressions]
     else:
-        return [col.name for col in idx.columns]
+        return [getattr(col, "name", None) for col in idx.columns]
 
 def _compare_indexes_and_uniques(schema, tname, object_filters, conn_table,
             metadata_table, diffs, autogen_context, inspector):
@@ -192,26 +212,29 @@ def _compare_indexes_and_uniques(schema, tname, object_filters, conn_table,
     )
     metadata_indexes = set(metadata_table.indexes)
 
-    # 1b. ... and from connection
-    if conn_table is not None and hasattr(inspector, "get_unique_constraints"):
+    conn_uniques = conn_indexes = frozenset()
+
+    supports_unique_constraints = False
+
+    if conn_table is not None:
+        # 1b. ... and from connection, if the table exists
+        if hasattr(inspector, "get_unique_constraints"):
+            try:
+                conn_uniques = inspector.get_unique_constraints(
+                                                tname, schema=schema)
+                supports_unique_constraints = True
+            except NotImplementedError:
+                pass
         try:
-            conn_uniques = inspector.get_unique_constraints(tname)
-        except (NotImplementedError, NoSuchTableError):
-            conn_uniques = []
-    else:
-        conn_uniques = []
+            conn_indexes = inspector.get_indexes(tname, schema=schema)
+        except NotImplementedError:
+            pass
 
-
-    try:
-        conn_indexes = inspector.get_indexes(tname)
-    except NoSuchTableError:
-        conn_indexes = []
-
-    # 2. convert conn-level objects from raw inspector records
-    # into schema objects
-    conn_uniques = set(_make_unique_constraint(uq_def, conn_table)
-                                    for uq_def in conn_uniques)
-    conn_indexes = set(_make_index(ix, conn_table) for ix in conn_indexes)
+        # 2. convert conn-level objects from raw inspector records
+        # into schema objects
+        conn_uniques = set(_make_unique_constraint(uq_def, conn_table)
+                                        for uq_def in conn_uniques)
+        conn_indexes = set(_make_index(ix, conn_table) for ix in conn_indexes)
 
     # 3. give the dialect a chance to omit indexes and constraints that
     # we know are either added implicitly by the DB or that the DB
@@ -282,6 +305,10 @@ def _compare_indexes_and_uniques(schema, tname, object_filters, conn_table,
                     ])
             )
         else:
+            if not supports_unique_constraints:
+                # can't report unique indexes as added if we don't
+                # detect them
+                return
             if is_create_table:
                 # unique constraints are created inline with table defs
                 return
@@ -294,6 +321,12 @@ def _compare_indexes_and_uniques(schema, tname, object_filters, conn_table,
 
     def obj_removed(obj):
         if obj.is_index:
+            if obj.is_unique and not supports_unique_constraints:
+                # many databases double up unique constraints
+                # as unique indexes.  without that list we can't
+                # be sure what we're doing here
+                return
+
             diffs.append(("remove_index", obj.const))
             log.info("Detected removed index '%s' on '%s'", obj.name, tname)
         else:
@@ -316,7 +349,6 @@ def _compare_indexes_and_uniques(schema, tname, object_filters, conn_table,
             diffs.append(("remove_constraint", old.const))
             diffs.append(("add_constraint", new.const))
 
-
     for added_name in sorted(set(metadata_names).difference(conn_names)):
         obj = metadata_names[added_name]
         obj_added(obj)
