@@ -1,5 +1,3 @@
-from contextlib import contextmanager
-
 from .operations import Operations
 from .migration import MigrationContext
 from . import util
@@ -60,7 +58,6 @@ class EnvironmentContext(object):
     """
 
     _migration_context = None
-    _default_opts = None
 
     config = None
     """An instance of :class:`.Config` representing the
@@ -87,8 +84,6 @@ class EnvironmentContext(object):
         self.config = config
         self.script = script
         self.context_opts = kw
-        if self._default_opts:
-            self.context_opts.update(self._default_opts)
 
     def __enter__(self):
         """Establish a context which provides a
@@ -246,7 +241,10 @@ class EnvironmentContext(object):
             :attr:`.Config.cmd_opts`
 
         """
-        value = self.config.cmd_opts.x or []
+        if self.config.cmd_opts is not None:
+            value = self.config.cmd_opts.x or []
+        else:
+            value = []
         if as_dictionary:
             value = dict(
                         arg.split('=', 1) for arg in value
@@ -258,6 +256,7 @@ class EnvironmentContext(object):
             url=None,
             dialect_name=None,
             transactional_ddl=None,
+            transaction_per_migration=False,
             output_buffer=None,
             starting_rev=None,
             tag=None,
@@ -273,6 +272,7 @@ class EnvironmentContext(object):
             downgrade_token="downgrades",
             alembic_module_prefix="op.",
             sqlalchemy_module_prefix="sa.",
+            user_module_prefix=None,
             **kw
         ):
         """Configure a :class:`.MigrationContext` within this
@@ -321,6 +321,12 @@ class EnvironmentContext(object):
          DDL on or off;
          this otherwise defaults to whether or not the dialect in
          use supports it.
+        :param transaction_per_migration: if True, nest each migration script
+         in a transaction rather than the full series of migrations to
+         run.
+
+         .. versionadded:: 0.6.5
+
         :param output_buffer: a file-like object that will be used
          for textual output
          when the ``--sql`` option is used to generate SQL scripts.
@@ -382,7 +388,13 @@ class EnvironmentContext(object):
                 # return True if the types are different,
                 # False if not, or None to allow the default implementation
                 # to compare these types
-                pass
+                return None
+
+            context.configure(
+                # ...
+                compare_type = my_compare_type
+            )
+
 
          ``inspected_column`` is a :class:`sqlalchemy.schema.Column` as returned by
          :meth:`sqlalchemy.engine.reflection.Inspector.reflecttable`, whereas
@@ -392,6 +404,10 @@ class EnvironmentContext(object):
          A return value of ``None`` indicates to allow default type
          comparison to proceed.
 
+         .. seealso::
+
+            :paramref:`.EnvironmentContext.configure.compare_server_default`
+
         :param compare_server_default: Indicates server default comparison
          behavior during
          an autogenerate operation.  Defaults to ``False`` which disables
@@ -413,7 +429,12 @@ class EnvironmentContext(object):
                 # return True if the defaults are different,
                 # False if not, or None to allow the default implementation
                 # to compare these defaults
-                pass
+                return None
+
+            context.configure(
+                # ...
+                compare_server_default = my_compare_server_default
+            )
 
          ``inspected_column`` is a dictionary structure as returned by
          :meth:`sqlalchemy.engine.reflection.Inspector.get_columns`, whereas
@@ -426,6 +447,10 @@ class EnvironmentContext(object):
          execute
          the two defaults on the database side to compare for equivalence.
 
+         .. seealso::
+
+            :paramref:`.EnvironmentContext.configure.compare_type`
+
         :param include_object: A callable function which is given
          the chance to return ``True`` or ``False`` for any object,
          indicating if the given object should be considered in the
@@ -439,8 +464,7 @@ class EnvironmentContext(object):
          * ``name``: the name of the object. This is typically available
            via ``object.name``.
          * ``type``: a string describing the type of object; currently
-           ``"table"`` or ``"column"``, but will include other types in a
-           future release
+           ``"table"`` or ``"column"``
          * ``reflected``: ``True`` if the given object was produced based on
            table reflection, ``False`` if it's from a local :class:`.MetaData`
            object.
@@ -462,22 +486,27 @@ class EnvironmentContext(object):
                 include_object = include_object
             )
 
-         The ``include_object`` filter will be expanded in a future release
-         to also receive type, constraint, and default objects.
+         :paramref:`.EnvironmentContext.configure.include_object` can also
+         be used to filter on specific schemas to include or omit, when
+         the :paramref:`.EnvironmentContext.configure.include_schemas`
+         flag is set to ``True``.   The :attr:`.Table.schema` attribute
+         on each :class:`.Table` object reflected will indicate the name of the
+         schema from which the :class:`.Table` originates.
 
          .. versionadded:: 0.6.0
 
          .. seealso::
 
-            ``include_schemas``, ``include_symbol``
-
+            :paramref:`.EnvironmentContext.configure.include_schemas`
 
         :param include_symbol: A callable function which, given a table name
          and schema name (may be ``None``), returns ``True`` or ``False``, indicating
          if the given table should be considered in the autogenerate sweep.
 
-         .. deprecated:: 0.6.0 ``include_symbol`` is superceded by the
-            more generic ``include_object`` parameter.
+         .. deprecated:: 0.6.0 :paramref:`.EnvironmentContext.configure.include_symbol`
+            is superceded by the more generic
+            :paramref:`.EnvironmentContext.configure.include_object`
+            parameter.
 
          E.g.::
 
@@ -489,40 +518,26 @@ class EnvironmentContext(object):
                 include_symbol = include_symbol
             )
 
-         To limit autogenerate to a certain set of schemas when using the
-         ``include_schemas`` option::
-
-            def include_symbol(tablename, schema):
-                return schema in (None, "schema1", "schema2")
-
-            context.configure(
-                # ...
-                include_schemas = True,
-                include_symbol = include_symbol
-            )
-
-         .. versionadded:: 0.3.6
-
-         .. versionchanged:: 0.4.0  the ``include_symbol`` callable must now
-            also accept a "schema" argument, which may be None.
-
          .. seealso::
 
-            ``include_schemas``, ``include_object``
+            :paramref:`.EnvironmentContext.configure.include_schemas`
+
+            :paramref:`.EnvironmentContext.configure.include_object`
 
         :param include_schemas: If True, autogenerate will scan across
          all schemas located by the SQLAlchemy
          :meth:`~sqlalchemy.engine.reflection.Inspector.get_schema_names`
          method, and include all differences in tables found across all
          those schemas.  When using this option, you may want to also
-         use the ``include_symbol`` option to specify a callable which
+         use the :paramref:`.EnvironmentContext.configure.include_object`
+         option to specify a callable which
          can filter the tables/schemas that get included.
 
          .. versionadded :: 0.4.0
 
          .. seealso::
 
-            ``include_symbol``, ``include_object``
+            :paramref:`.EnvironmentContext.configure.include_object`
 
         :param render_item: Callable that can be used to override how
          any schema item, i.e. column, constraint, type,
@@ -545,12 +560,16 @@ class EnvironmentContext(object):
                 render_item = my_render_column
             )
 
-         Available values for the type string include: ``column``,
-         ``primary_key``, ``foreign_key``, ``unique``, ``check``,
-         ``type``, ``server_default``.
+         Available values for the type string include: ``"column"``,
+         ``"primary_key"``, ``"foreign_key"``, ``"unique"``, ``"check"``,
+         ``"type"``, ``"server_default"``.
 
          .. versionadded:: 0.5.0
 
+         .. seealso::
+
+            :ref:`autogen_render_types`
+
         :param upgrade_token: When autogenerate completes, the text of the
          candidate upgrade operations will be present in this template
          variable when ``script.py.mako`` is rendered.  Defaults to
@@ -575,6 +594,20 @@ class EnvironmentContext(object):
          will render them using the dialect module name, i.e. ``mssql.BIT()``,
          ``postgresql.UUID()``.
 
+        :param user_module_prefix: When autogenerate refers to a SQLAlchemy
+         type (e.g. :class:`.TypeEngine`) where the module name is not
+         under the ``sqlalchemy`` namespace, this prefix will be used
+         within autogenerate, if non-``None``; if left at its default of
+         ``None``, the
+         :paramref:`.EnvironmentContext.configure.sqlalchemy_module_prefix`
+         is used instead.
+
+         .. versionadded:: 0.6.3 added
+            :paramref:`.EnvironmentContext.configure.user_module_prefix`
+
+         .. seealso::
+
+            :ref:`autogen_module_prefix`
 
         Parameters specific to individual backends:
 
@@ -604,6 +637,7 @@ class EnvironmentContext(object):
             opts['tag'] = tag
         if template_args and 'template_args' in opts:
             opts['template_args'].update(template_args)
+        opts["transaction_per_migration"] = transaction_per_migration
         opts['target_metadata'] = target_metadata
         opts['include_symbol'] = include_symbol
         opts['include_object'] = include_object
@@ -612,6 +646,7 @@ class EnvironmentContext(object):
         opts['downgrade_token'] = downgrade_token
         opts['sqlalchemy_module_prefix'] = sqlalchemy_module_prefix
         opts['alembic_module_prefix'] = alembic_module_prefix
+        opts['user_module_prefix'] = user_module_prefix
         if render_item is not None:
             opts['render_item'] = render_item
         if compare_type is not None:
@@ -619,6 +654,7 @@ class EnvironmentContext(object):
         if compare_server_default is not None:
             opts['compare_server_default'] = compare_server_default
         opts['script'] = self.script
+
         opts.update(kw)
 
         self._migration_context = MigrationContext.configure(
@@ -677,6 +713,7 @@ class EnvironmentContext(object):
         """
         self.get_context().impl.static_output(text)
 
+
     def begin_transaction(self):
         """Return a context manager that will
         enclose an operation within a "transaction",
@@ -720,20 +757,9 @@ class EnvironmentContext(object):
         mode.
 
         """
-        if not self.is_transactional_ddl():
-            @contextmanager
-            def do_nothing():
-                yield
-            return do_nothing()
-        elif self.is_offline_mode():
-            @contextmanager
-            def begin_commit():
-                self.get_context().impl.emit_begin()
-                yield
-                self.get_context().impl.emit_commit()
-            return begin_commit()
-        else:
-            return self.get_bind().begin()
+
+        return self.get_context().begin_transaction()
+
 
     def get_context(self):
         """Return the current :class:`.MigrationContext` object.
