@@ -1,12 +1,11 @@
-from __future__ import with_statement
-
-import os
-from alembic import util
-import shutil
-import re
 import datetime
+import os
+import re
+import shutil
+from . import util
 
-_rev_file = re.compile(r'.*\.py$')
+_sourceless_rev_file = re.compile(r'(.*\.py)(c|o)?$')
+_only_source_rev_file = re.compile(r'(.*\.py)$')
 _legacy_rev = re.compile(r'([a-f0-9]+)\.py$')
 _mod_def_re = re.compile(r'(upgrade|downgrade)_([a-z0-9]+)')
 _slug_re = re.compile(r'\w+')
@@ -32,10 +31,14 @@ class ScriptDirectory(object):
 
 
     """
-    def __init__(self, dir, file_template=_default_file_template):
+    def __init__(self, dir, file_template=_default_file_template,
+                    truncate_slug_length=40,
+                    sourceless=False):
         self.dir = dir
         self.versions = os.path.join(self.dir, 'versions')
         self.file_template = file_template
+        self.truncate_slug_length = truncate_slug_length or 40
+        self.sourceless = sourceless
 
         if not os.access(dir, os.F_OK):
             raise util.CommandError("Path doesn't exist: %r.  Please use "
@@ -55,22 +58,29 @@ class ScriptDirectory(object):
         if script_location is None:
             raise util.CommandError("No 'script_location' key "
                                     "found in configuration.")
+        truncate_slug_length = config.get_main_option("truncate_slug_length")
+        if truncate_slug_length is not None:
+            truncate_slug_length = int(truncate_slug_length)
         return ScriptDirectory(
                     util.coerce_resource_to_filename(script_location),
-                    file_template = config.get_main_option(
+                    file_template=config.get_main_option(
                                         'file_template',
-                                        _default_file_template)
+                                        _default_file_template),
+                    truncate_slug_length=truncate_slug_length,
+                    sourceless=config.get_main_option("sourceless") == "true"
                     )
 
-    def walk_revisions(self):
+    def walk_revisions(self, base="base", head="head"):
         """Iterate through all revisions.
 
         This is actually a breadth-first tree traversal,
         with leaf nodes being heads.
 
         """
-        heads = set(self.get_heads())
-        base = self.get_revision("base")
+        if head == "head":
+            heads = set(self.get_heads())
+        else:
+            heads = set([head])
         while heads:
             todo = set(heads)
             heads = set()
@@ -168,14 +178,16 @@ class ScriptDirectory(object):
     def _upgrade_revs(self, destination, current_rev):
         revs = self.iterate_revisions(destination, current_rev)
         return [
-            (script.module.upgrade, script.down_revision, script.revision)
+            (script.module.upgrade, script.down_revision, script.revision,
+                script.doc)
             for script in reversed(list(revs))
             ]
 
     def _downgrade_revs(self, destination, current_rev):
         revs = self.iterate_revisions(current_rev, destination)
         return [
-            (script.module.downgrade, script.revision, script.down_revision)
+            (script.module.downgrade, script.revision, script.down_revision,
+                script.doc)
             for script in revs
             ]
 
@@ -198,7 +210,7 @@ class ScriptDirectory(object):
     def _revision_map(self):
         map_ = {}
         for file_ in os.listdir(self.versions):
-            script = Script._from_filename(self.versions, file_)
+            script = Script._from_filename(self, self.versions, file_)
             if script is None:
                 continue
             if script.revision in map_:
@@ -218,7 +230,9 @@ class ScriptDirectory(object):
         return map_
 
     def _rev_path(self, rev_id, message, create_date):
-        slug = "_".join(_slug_re.findall(message or "")).lower()[0:20]
+        slug = "_".join(_slug_re.findall(message or "")).lower()
+        if len(slug) > self.truncate_slug_length:
+            slug = slug[:self.truncate_slug_length].rsplit('_', 1)[0] + '_'
         filename = "%s.py" % (
             self.file_template % {
                 'rev': rev_id,
@@ -244,7 +258,12 @@ class ScriptDirectory(object):
         """
         current_heads = self.get_heads()
         if len(current_heads) > 1:
-            raise util.CommandError("Only a single head supported so far...")
+            raise util.CommandError('Only a single head is supported. The '
+                'script directory has multiple heads (due to branching), which '
+                'must be resolved by manually editing the revision files to '
+                'form a linear sequence. Run `alembic branches` to see the '
+                'divergence(s).')
+
         if current_heads:
             return current_heads[0]
         else:
@@ -282,7 +301,7 @@ class ScriptDirectory(object):
 
         """
         for script in self._revision_map.values():
-            if script.down_revision is None \
+            if script and script.down_revision is None \
                 and script.revision in self._revision_map:
                 return script.revision
         else:
@@ -333,7 +352,7 @@ class ScriptDirectory(object):
             **kw
         )
         if refresh:
-            script = Script._from_path(path)
+            script = Script._from_path(self, path)
             self._revision_map[script.revision] = script
             if script.down_revision:
                 self._revision_map[script.down_revision].\
@@ -374,7 +393,20 @@ class Script(object):
     @property
     def doc(self):
         """Return the docstring given in the script."""
-        return re.split(r"\n\n", self.module.__doc__)[0]
+
+        return re.split("\n\n", self.longdoc)[0]
+
+    @property
+    def longdoc(self):
+        """Return the docstring given in the script."""
+
+        doc = self.module.__doc__
+        if doc:
+            if hasattr(self.module, "_alembic_source_encoding"):
+                doc = doc.decode(self.module._alembic_source_encoding)
+            return doc.strip()
+        else:
+            return ""
 
     def add_nextrev(self, rev):
         self.nextrev = self.nextrev.union([rev])
@@ -402,6 +434,24 @@ class Script(object):
         """
         return len(self.nextrev) > 1
 
+    @property
+    def log_entry(self):
+        return \
+            "Rev: %s%s%s\n" \
+            "Parent: %s\n" \
+            "Path: %s\n" \
+            "\n%s\n" % (
+                self.revision,
+                " (head)" if self.is_head else "",
+                " (branchpoint)" if self.is_branch_point else "",
+                self.down_revision,
+                self.path,
+                "\n".join(
+                    "    %s" % para
+                    for para in self.longdoc.splitlines()
+                )
+            )
+
     def __str__(self):
         return "%s -> %s%s%s, %s" % (
                         self.down_revision,
@@ -411,15 +461,40 @@ class Script(object):
                         self.doc)
 
     @classmethod
-    def _from_path(cls, path):
+    def _from_path(cls, scriptdir, path):
         dir_, filename = os.path.split(path)
-        return cls._from_filename(dir_, filename)
+        return cls._from_filename(scriptdir, dir_, filename)
 
     @classmethod
-    def _from_filename(cls, dir_, filename):
-        if not _rev_file.match(filename):
+    def _from_filename(cls, scriptdir, dir_, filename):
+        if scriptdir.sourceless:
+            py_match = _sourceless_rev_file.match(filename)
+        else:
+            py_match = _only_source_rev_file.match(filename)
+
+        if not py_match:
             return None
+
+        py_filename = py_match.group(1)
+
+        if scriptdir.sourceless:
+            is_c = py_match.group(2) == 'c'
+            is_o = py_match.group(2) == 'o'
+        else:
+            is_c = is_o = False
+
+        if is_o or is_c:
+            py_exists = os.path.exists(os.path.join(dir_, py_filename))
+            pyc_exists = os.path.exists(os.path.join(dir_, py_filename + "c"))
+
+            # prefer .py over .pyc because we'd like to get the
+            # source encoding; prefer .pyc over .pyo because we'd like to
+            # have the docstrings which a -OO file would not have
+            if py_exists or is_o and pyc_exists:
+                return None
+
         module = util.load_python_file(dir_, filename)
+
         if not hasattr(module, "revision"):
             # attempt to get the revision id from the script name,
             # this for legacy only
