@@ -12,7 +12,7 @@ from sqlalchemy.dialects import mysql, postgresql
 from sqlalchemy.sql import and_, column, literal_column
 
 from alembic import autogenerate, util, compat
-from . import eq_, eq_ignore_whitespace
+from . import eq_, eq_ignore_whitespace, requires_092, requires_09, requires_094
 
 py3k = sys.version_info >= (3, )
 
@@ -68,7 +68,7 @@ class AutogenRenderTest(TestCase):
         idx = Index('test_active_code_idx', t.c.active, t.c.code)
         eq_ignore_whitespace(
             autogenerate.render._add_index(idx, self.autogen_context),
-            "op.create_index('test_active_code_idx', 'CamelSchema.test', "
+            "op.create_index('test_active_code_idx', 'test', "
             "['active', 'code'], unique=False, schema='CamelSchema')"
         )
 
@@ -127,7 +127,25 @@ class AutogenRenderTest(TestCase):
         idx = Index('test_active_code_idx', t.c.active, t.c.code)
         eq_ignore_whitespace(
             autogenerate.render._drop_index(idx, self.autogen_context),
-            "op.drop_index('test_active_code_idx', 'test')"
+            "op.drop_index('test_active_code_idx', table_name='test')"
+        )
+
+    def test_drop_index_schema(self):
+        """
+        autogenerate.render._drop_index using schema
+        """
+        m = MetaData()
+        t = Table('test', m,
+            Column('id', Integer, primary_key=True),
+            Column('active', Boolean()),
+            Column('code', String(255)),
+            schema='CamelSchema'
+        )
+        idx = Index('test_active_code_idx', t.c.active, t.c.code)
+        eq_ignore_whitespace(
+            autogenerate.render._drop_index(idx, self.autogen_context),
+            "op.drop_index('test_active_code_idx', " +
+                          "table_name='test', schema='CamelSchema')"
         )
 
     def test_add_unique_constraint(self):
@@ -146,6 +164,23 @@ class AutogenRenderTest(TestCase):
             "op.create_unique_constraint('uq_test_code', 'test', ['code'])"
         )
 
+    def test_add_unique_constraint_schema(self):
+        """
+        autogenerate.render._add_unique_constraint using schema
+        """
+        m = MetaData()
+        t = Table('test', m,
+            Column('id', Integer, primary_key=True),
+            Column('active', Boolean()),
+            Column('code', String(255)),
+            schema='CamelSchema'
+        )
+        uq = UniqueConstraint(t.c.code, name='uq_test_code')
+        eq_ignore_whitespace(
+            autogenerate.render._add_unique_constraint(uq, self.autogen_context),
+            "op.create_unique_constraint('uq_test_code', 'test', ['code'], schema='CamelSchema')"
+        )
+
     def test_drop_constraint(self):
         """
         autogenerate.render._drop_constraint
@@ -162,6 +197,23 @@ class AutogenRenderTest(TestCase):
             "op.drop_constraint('uq_test_code', 'test')"
         )
 
+    def test_drop_constraint_schema(self):
+        """
+        autogenerate.render._drop_constraint using schema
+        """
+        m = MetaData()
+        t = Table('test', m,
+            Column('id', Integer, primary_key=True),
+            Column('active', Boolean()),
+            Column('code', String(255)),
+            schema='CamelSchema'
+        )
+        uq = UniqueConstraint(t.c.code, name='uq_test_code')
+        eq_ignore_whitespace(
+            autogenerate.render._drop_constraint(uq, self.autogen_context),
+            "op.drop_constraint('uq_test_code', 'test', schema='CamelSchema')"
+        )
+
     def test_render_table_upgrade(self):
         m = MetaData()
         t = Table('test', m,
@@ -263,7 +315,7 @@ class AutogenRenderTest(TestCase):
         t = Table('test', m,
             Column('id', Integer, primary_key=True),
             Column('q', Integer, ForeignKey('bar.address.id')),
-            postgresql_arg1="some_arg", mysql_engine="InnoDB"
+            sqlite_autoincrement=True, mysql_engine="InnoDB"
         )
         eq_ignore_whitespace(
             autogenerate.render._add_table(t, self.autogen_context),
@@ -272,7 +324,7 @@ class AutogenRenderTest(TestCase):
             "sa.Column('q', sa.Integer(), nullable=True),"
             "sa.ForeignKeyConstraint(['q'], ['bar.address.id'], ),"
             "sa.PrimaryKeyConstraint('id'),"
-            "mysql_engine='InnoDB',postgresql_arg1='some_arg')"
+            "mysql_engine='InnoDB',sqlite_autoincrement=True)"
         )
 
     def test_render_drop_table(self):
@@ -524,6 +576,7 @@ render:primary_key\n)"""
             "sa.CheckConstraint('im a constraint', name='cc1')"
         )
 
+
     def test_render_check_constraint_sqlexpr(self):
         c = column('c')
         five = literal_column('5')
@@ -560,20 +613,253 @@ render:primary_key\n)"""
             "existing_server_default='5')"
         )
 
+
+
     def test_render_enum(self):
         eq_ignore_whitespace(
             autogenerate.render._repr_type(
-                        "sa.",
                         Enum("one", "two", "three", name="myenum"),
                         self.autogen_context),
             "sa.Enum('one', 'two', 'three', name='myenum')"
         )
         eq_ignore_whitespace(
             autogenerate.render._repr_type(
-                        "sa.",
                         Enum("one", "two", "three"),
                         self.autogen_context),
             "sa.Enum('one', 'two', 'three')"
         )
 
-# TODO: tests for dialect-specific type rendering + imports
+    def test_repr_plain_sqla_type(self):
+        type_ = Integer()
+        autogen_context = {
+            'opts': {
+                'sqlalchemy_module_prefix': 'sa.',
+                'alembic_module_prefix': 'op.',
+            },
+            'dialect': mysql.dialect()
+        }
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(type_, autogen_context),
+            "sa.Integer()"
+        )
+
+    def test_repr_user_type_user_prefix_None(self):
+        from sqlalchemy.types import UserDefinedType
+        class MyType(UserDefinedType):
+            def get_col_spec(self):
+                return "MYTYPE"
+
+        type_ = MyType()
+        autogen_context = {
+            'opts': {
+                'sqlalchemy_module_prefix': 'sa.',
+                'alembic_module_prefix': 'op.',
+                'user_module_prefix': None
+            },
+            'dialect': mysql.dialect()
+        }
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(type_, autogen_context),
+            "sa.MyType()"
+        )
+
+    def test_repr_user_type_user_prefix_present(self):
+        from sqlalchemy.types import UserDefinedType
+        class MyType(UserDefinedType):
+            def get_col_spec(self):
+                return "MYTYPE"
+
+        type_ = MyType()
+        autogen_context = {
+            'opts': {
+                'sqlalchemy_module_prefix': 'sa.',
+                'alembic_module_prefix': 'op.',
+                'user_module_prefix': 'user.',
+            },
+            'dialect': mysql.dialect()
+        }
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(type_, autogen_context),
+            "user.MyType()"
+        )
+
+    @requires_09
+    def test_repr_dialect_type(self):
+        from sqlalchemy.dialects.mysql import VARCHAR
+
+        type_ = VARCHAR(20, charset='utf8', national=True)
+        autogen_context = {
+            'opts': {
+                'sqlalchemy_module_prefix': 'sa.',
+                'alembic_module_prefix': 'op.',
+                'user_module_prefix': None,
+            },
+            'imports': set(),
+            'dialect': mysql.dialect()
+        }
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(type_, autogen_context),
+            "mysql.VARCHAR(charset='utf8', national=True, length=20)"
+        )
+        eq_(autogen_context['imports'],
+                set(['from sqlalchemy.dialects import mysql'])
+            )
+
+class RenderNamingConventionTest(TestCase):
+
+    @classmethod
+    @requires_094
+    def setup_class(cls):
+        cls.autogen_context = {
+            'opts': {
+                'sqlalchemy_module_prefix': 'sa.',
+                'alembic_module_prefix': 'op.',
+            },
+            'dialect': postgresql.dialect()
+        }
+
+
+    def setUp(self):
+
+        convention = {
+          "ix": 'ix_%(custom)s_%(column_0_label)s',
+          "uq": "uq_%(custom)s_%(table_name)s_%(column_0_name)s",
+          "ck": "ck_%(custom)s_%(table_name)s",
+          "fk": "fk_%(custom)s_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
+          "pk": "pk_%(custom)s_%(table_name)s",
+          "custom": lambda const, table: "ct"
+        }
+
+        self.metadata = MetaData(
+                            naming_convention=convention
+                        )
+
+    def test_schema_type_boolean(self):
+        t = Table('t', self.metadata, Column('c', Boolean(name='xyz')))
+        eq_ignore_whitespace(
+            autogenerate.render._add_column(
+                    None, "t", t.c.c,
+                        self.autogen_context),
+            "op.add_column('t', "
+                "sa.Column('c', sa.Boolean(name='xyz'), nullable=True))"
+        )
+
+    def test_explicit_unique_constraint(self):
+        t = Table('t', self.metadata, Column('c', Integer))
+        eq_ignore_whitespace(
+            autogenerate.render._render_unique_constraint(
+                UniqueConstraint(t.c.c, deferrable='XYZ'),
+                self.autogen_context
+            ),
+            "sa.UniqueConstraint('c', deferrable='XYZ', name=op.f('uq_ct_t_c'))"
+        )
+
+    def test_explicit_named_unique_constraint(self):
+        t = Table('t', self.metadata, Column('c', Integer))
+        eq_ignore_whitespace(
+            autogenerate.render._render_unique_constraint(
+                UniqueConstraint(t.c.c, name='q'),
+                self.autogen_context
+            ),
+            "sa.UniqueConstraint('c', name='q')"
+        )
+
+    def test_render_add_index(self):
+        t = Table('test', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('active', Boolean()),
+            Column('code', String(255)),
+        )
+        idx = Index(None, t.c.active, t.c.code)
+        eq_ignore_whitespace(
+            autogenerate.render._add_index(idx, self.autogen_context),
+            "op.create_index(op.f('ix_ct_test_active'), 'test', "
+            "['active', 'code'], unique=False)"
+        )
+
+    def test_render_drop_index(self):
+        t = Table('test', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('active', Boolean()),
+            Column('code', String(255)),
+        )
+        idx = Index(None, t.c.active, t.c.code)
+        eq_ignore_whitespace(
+            autogenerate.render._drop_index(idx, self.autogen_context),
+            "op.drop_index(op.f('ix_ct_test_active'), table_name='test')"
+        )
+
+    def test_render_add_index_schema(self):
+        t = Table('test', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('active', Boolean()),
+            Column('code', String(255)),
+            schema='CamelSchema'
+        )
+        idx = Index(None, t.c.active, t.c.code)
+        eq_ignore_whitespace(
+            autogenerate.render._add_index(idx, self.autogen_context),
+            "op.create_index(op.f('ix_ct_CamelSchema_test_active'), 'test', "
+            "['active', 'code'], unique=False, schema='CamelSchema')"
+        )
+
+
+    def test_implicit_unique_constraint(self):
+        t = Table('t', self.metadata, Column('c', Integer, unique=True))
+        uq = [c for c in t.constraints if isinstance(c, UniqueConstraint)][0]
+        eq_ignore_whitespace(
+            autogenerate.render._render_unique_constraint(uq,
+                self.autogen_context
+            ),
+            "sa.UniqueConstraint('c', name=op.f('uq_ct_t_c'))"
+        )
+
+    def test_inline_pk_constraint(self):
+        t = Table('t', self.metadata, Column('c', Integer, primary_key=True))
+        eq_ignore_whitespace(
+            autogenerate.render._add_table(t, self.autogen_context),
+            "op.create_table('t',sa.Column('c', sa.Integer(), nullable=False),"
+                "sa.PrimaryKeyConstraint('c', name=op.f('pk_ct_t')))"
+        )
+
+    def test_inline_ck_constraint(self):
+        t = Table('t', self.metadata, Column('c', Integer), CheckConstraint("c > 5"))
+        eq_ignore_whitespace(
+            autogenerate.render._add_table(t, self.autogen_context),
+            "op.create_table('t',sa.Column('c', sa.Integer(), nullable=True),"
+                "sa.CheckConstraint('c > 5', name=op.f('ck_ct_t')))"
+        )
+
+    def test_inline_fk(self):
+        t = Table('t', self.metadata, Column('c', Integer, ForeignKey('q.id')))
+        eq_ignore_whitespace(
+            autogenerate.render._add_table(t, self.autogen_context),
+            "op.create_table('t',sa.Column('c', sa.Integer(), nullable=True),"
+                "sa.ForeignKeyConstraint(['c'], ['q.id'], name=op.f('fk_ct_t_c_q')))"
+        )
+
+    def test_render_check_constraint_renamed(self):
+        """test that constraints from autogenerate render with
+        the naming convention name explicitly.  These names should
+        be frozen into the migration scripts so that they remain
+        the same if the application's naming convention changes.
+
+        However, op.create_table() and others need to be careful that
+        these don't double up when the "%(constraint_name)s" token is
+        used.
+
+        """
+        m1 = MetaData(naming_convention={"ck": "ck_%(table_name)s_%(constraint_name)s"})
+        ck = CheckConstraint("im a constraint", name="cc1")
+        Table('t', m1, Column('x'), ck)
+
+        eq_ignore_whitespace(
+            autogenerate.render._render_check_constraint(
+                ck,
+                self.autogen_context
+            ),
+            "sa.CheckConstraint('im a constraint', name=op.f('ck_t_cc1'))"
+        )
