@@ -8,20 +8,16 @@ from sqlalchemy import MetaData, Column, Table, Integer, String, Text, \
     TypeDecorator, CheckConstraint, Unicode, Enum,\
     UniqueConstraint, Boolean, ForeignKeyConstraint,\
     PrimaryKeyConstraint, Index, func
-from sqlalchemy.types import NULLTYPE, TIMESTAMP
-from sqlalchemy.dialects import mysql, postgresql
+from sqlalchemy.types import NULLTYPE
 from sqlalchemy.engine.reflection import Inspector
-from sqlalchemy.sql import and_, column, literal_column
 
-from alembic import autogenerate, util, compat
+from alembic import autogenerate
 from alembic.migration import MigrationContext
 from . import staging_env, sqlite_db, clear_staging_env, eq_, \
-        eq_ignore_whitespace, db_for_dialect
+        db_for_dialect
 
 py3k = sys.version_info >= (3, )
 
-
-
 names_in_this_test = set()
 def _default_include_object(obj, name, type_, reflected, compare_to):
     if type_ == "table":
@@ -77,6 +73,55 @@ class AutogenTest(object):
         cls.m1.drop_all(cls.bind)
         clear_staging_env()
 
+class AutogenFixtureTest(object):
+    def _fixture(self, m1, m2, include_schemas=False):
+        self.metadata, model_metadata = m1, m2
+        self.metadata.create_all(self.bind)
+
+        with self.bind.connect() as conn:
+            self.context = context = MigrationContext.configure(
+                connection=conn,
+                opts={
+                    'compare_type': True,
+                    'compare_server_default': True,
+                    'target_metadata': model_metadata,
+                    'upgrade_token': "upgrades",
+                    'downgrade_token': "downgrades",
+                    'alembic_module_prefix': 'op.',
+                    'sqlalchemy_module_prefix': 'sa.',
+                }
+            )
+
+            connection = context.bind
+            autogen_context = {
+                'imports': set(),
+                'connection': connection,
+                'dialect': connection.dialect,
+                'context': context
+                }
+            diffs = []
+            autogenerate._produce_net_changes(connection, model_metadata, diffs,
+                                              autogen_context,
+                                              object_filters=_default_object_filters,
+                                              include_schemas=include_schemas
+                                        )
+            return diffs
+
+    reports_unnamed_constraints = False
+
+    def setUp(self):
+        staging_env()
+        self.bind = self._get_bind()
+
+    def tearDown(self):
+        if hasattr(self, 'metadata'):
+            self.metadata.drop_all(self.bind)
+        clear_staging_env()
+
+    @classmethod
+    def _get_bind(cls):
+        return sqlite_db()
+
 
 class AutogenCrossSchemaTest(AutogenTest, TestCase):
     @classmethod
@@ -177,6 +222,62 @@ class AutogenCrossSchemaTest(AutogenTest, TestCase):
         eq_(diffs[0][1].schema, self.test_schema_name)
 
 
+class AutogenDefaultSchemaTest(AutogenFixtureTest, TestCase):
+    @classmethod
+    def _get_bind(cls):
+        cls.test_schema_name = "test_schema"
+        return db_for_dialect('postgresql')
+
+    def test_uses_explcit_schema_in_default_one(self):
+
+        default_schema = self.bind.dialect.default_schema_name
+
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('a', m1, Column('x', String(50)))
+        Table('a', m2, Column('x', String(50)), schema=default_schema)
+
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(diffs, [])
+
+    def test_uses_explcit_schema_in_default_two(self):
+
+        default_schema = self.bind.dialect.default_schema_name
+
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('a', m1, Column('x', String(50)))
+        Table('a', m2, Column('x', String(50)), schema=default_schema)
+        Table('a', m2, Column('y', String(50)), schema="test_schema")
+
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(len(diffs), 1)
+        eq_(diffs[0][0], "add_table")
+        eq_(diffs[0][1].schema, "test_schema")
+        eq_(diffs[0][1].c.keys(), ['y'])
+
+    def test_uses_explcit_schema_in_default_three(self):
+
+        default_schema = self.bind.dialect.default_schema_name
+
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('a', m1, Column('y', String(50)), schema="test_schema")
+
+        Table('a', m2, Column('x', String(50)), schema=default_schema)
+        Table('a', m2, Column('y', String(50)), schema="test_schema")
+
+
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(len(diffs), 1)
+        eq_(diffs[0][0], "add_table")
+        eq_(diffs[0][1].schema, default_schema)
+        eq_(diffs[0][1].c.keys(), ['x'])
+
+
 class ModelOne(object):
     schema = None
 
@@ -299,6 +400,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestCase):
         eq_(diffs[7][0][5], True)
         eq_(diffs[7][0][6], False)
 
+
     def test_render_nothing(self):
         context = MigrationContext.configure(
             connection=self.bind.connect(),
@@ -414,7 +516,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestCase):
                     assert obj.metadata is not self.m2
                 else:
                     assert obj.metadata is self.m2
-                return name in ("address", "order")
+                return name in ("address", "order", "user")
             elif type_ == "column":
                 if reflected:
                     assert obj.table.metadata is not self.m2
@@ -424,7 +526,6 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestCase):
             else:
                 return True
 
-
         context = MigrationContext.configure(
             connection=self.bind.connect(),
             opts={
@@ -440,12 +541,15 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestCase):
         )
         template_args = {}
         autogenerate._produce_migration_diffs(context, template_args, set())
+
         template_args['upgrades'] = template_args['upgrades'].replace("u'", "'")
         template_args['downgrades'] = template_args['downgrades'].\
                                         replace("u'", "'")
+        assert "op.create_table('item'" not in template_args['upgrades']
+        assert "op.create_table('item'" not in template_args['downgrades']
 
-        assert "alter_column('user'" not in template_args['upgrades']
-        assert "alter_column('user'" not in template_args['downgrades']
+        assert "alter_column('user'" in template_args['upgrades']
+        assert "alter_column('user'" in template_args['downgrades']
         assert "'street'" not in template_args['upgrades']
         assert "'street'" not in template_args['downgrades']
         assert "alter_column('order'" in template_args['upgrades']
@@ -638,7 +742,7 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestCase):
                server_default=None,
                existing_nullable=True,
                schema='%(schema)s')
-    op.add_column('user', sa.Column('pw', sa.VARCHAR(length=50), nullable=True), schema='%(schema)s')
+    op.add_column('user', sa.Column('pw', sa.VARCHAR(length=50), autoincrement=False, nullable=True), schema='%(schema)s')
     op.alter_column('order', 'amount',
                existing_type=sa.Numeric(precision=10, scale=2),
                type_=sa.NUMERIC(precision=8, scale=2),
@@ -657,353 +761,7 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestCase):
     ### end Alembic commands ###""" % {"schema": self.schema})
 
 
-class AutogenerateUniqueIndexTest(TestCase):
 
-    def test_index_flag_becomes_named_unique_constraint(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table('user', m1,
-            Column('id', Integer, primary_key=True),
-            Column('name', String(50), nullable=False, index=True),
-            Column('a1', String(10), server_default="x")
-        )
-
-        Table('user', m2,
-            Column('id', Integer, primary_key=True),
-            Column('name', String(50), nullable=False),
-            Column('a1', String(10), server_default="x"),
-            UniqueConstraint("name", name="uq_user_name")
-        )
-
-        diffs = self._fixture(m1, m2)
-
-        eq_(diffs[0][0], "add_constraint")
-        eq_(diffs[0][1].name, "uq_user_name")
-
-        eq_(diffs[1][0], "remove_index")
-        eq_(diffs[1][1].name, "ix_user_name")
-
-
-    def test_add_unique_constraint(self):
-        m1 = MetaData()
-        m2 = MetaData()
-        Table('address', m1,
-            Column('id', Integer, primary_key=True),
-            Column('email_address', String(100), nullable=False),
-            Column('qpr', String(10), index=True),
-        )
-        Table('address', m2,
-            Column('id', Integer, primary_key=True),
-            Column('email_address', String(100), nullable=False),
-            Column('qpr', String(10), index=True),
-            UniqueConstraint("email_address", name="uq_email_address")
-        )
-
-        diffs = self._fixture(m1, m2)
-        eq_(diffs[0][0], "add_constraint")
-        eq_(diffs[0][1].name, "uq_email_address")
-
-
-    def test_index_becomes_unique(self):
-        m1 = MetaData()
-        m2 = MetaData()
-        Table('order', m1,
-            Column('order_id', Integer, primary_key=True),
-            Column('amount', Numeric(10, 2), nullable=True),
-            Column('user_id', Integer),
-            UniqueConstraint('order_id', 'user_id',
-                name='order_order_id_user_id_unique'
-            ),
-            Index('order_user_id_amount_idx', 'user_id', 'amount')
-        )
-
-        Table('order', m2,
-            Column('order_id', Integer, primary_key=True),
-            Column('amount', Numeric(10, 2), nullable=True),
-            Column('user_id', Integer),
-            UniqueConstraint('order_id', 'user_id',
-                name='order_order_id_user_id_unique'
-            ),
-            Index('order_user_id_amount_idx', 'user_id', 'amount', unique=True),
-        )
-
-        diffs = self._fixture(m1, m2)
-        eq_(diffs[0][0], "remove_index")
-        eq_(diffs[0][1].name, "order_user_id_amount_idx")
-        eq_(diffs[0][1].unique, False)
-
-        eq_(diffs[1][0], "add_index")
-        eq_(diffs[1][1].name, "order_user_id_amount_idx")
-        eq_(diffs[1][1].unique, True)
-
-
-    def test_mismatch_db_named_col_flag(self):
-        m1 = MetaData()
-        m2 = MetaData()
-        Table('item', m1,
-                Column('x', Integer),
-                UniqueConstraint('x', name="db_generated_name")
-            )
-
-        # test mismatch between unique=True and
-        # named uq constraint
-        Table('item', m2,
-                Column('x', Integer, unique=True)
-            )
-
-        diffs = self._fixture(m1, m2)
-
-        eq_(diffs, [])
-
-    def test_new_table_added(self):
-        m1 = MetaData()
-        m2 = MetaData()
-        Table('extra', m2,
-                Column('foo', Integer, index=True),
-                Column('bar', Integer),
-                Index('newtable_idx', 'bar')
-            )
-
-        diffs = self._fixture(m1, m2)
-
-        eq_(diffs[0][0], "add_table")
-
-        eq_(diffs[1][0], "add_index")
-        eq_(diffs[1][1].name, "ix_extra_foo")
-
-        eq_(diffs[2][0], "add_index")
-        eq_(diffs[2][1].name, "newtable_idx")
-
-
-    def test_named_cols_changed(self):
-        m1 = MetaData()
-        m2 = MetaData()
-        Table('col_change', m1,
-                Column('x', Integer),
-                Column('y', Integer),
-                UniqueConstraint('x', name="nochange")
-            )
-        Table('col_change', m2,
-                Column('x', Integer),
-                Column('y', Integer),
-                UniqueConstraint('x', 'y', name="nochange")
-            )
-
-        diffs = self._fixture(m1, m2)
-
-        eq_(diffs[0][0], "remove_constraint")
-        eq_(diffs[0][1].name, "nochange")
-
-        eq_(diffs[1][0], "add_constraint")
-        eq_(diffs[1][1].name, "nochange")
-
-
-    def test_nothing_changed_one(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table('nothing_changed', m1,
-            Column('x', String(20), unique=True, index=True)
-            )
-
-        Table('nothing_changed', m2,
-            Column('x', String(20), unique=True, index=True)
-            )
-
-        diffs = self._fixture(m1, m2)
-        eq_(diffs, [])
-
-
-    def test_nothing_changed_two(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table('nothing_changed', m1,
-            Column('id1', Integer, primary_key=True),
-            Column('id2', Integer, primary_key=True),
-            Column('x', String(20), unique=True)
-            )
-        Table('nothing_changed_related', m1,
-            Column('id1', Integer),
-            Column('id2', Integer),
-            ForeignKeyConstraint(['id1', 'id2'],
-                    ['nothing_changed.id1', 'nothing_changed.id2'])
-            )
-
-        Table('nothing_changed', m2,
-            Column('id1', Integer, primary_key=True),
-            Column('id2', Integer, primary_key=True),
-            Column('x', String(20), unique=True)
-            )
-        Table('nothing_changed_related', m2,
-            Column('id1', Integer),
-            Column('id2', Integer),
-            ForeignKeyConstraint(['id1', 'id2'],
-                    ['nothing_changed.id1', 'nothing_changed.id2'])
-            )
-
-
-        diffs = self._fixture(m1, m2)
-        eq_(diffs, [])
-
-
-    def test_unnamed_cols_changed(self):
-        m1 = MetaData()
-        m2 = MetaData()
-        Table('col_change', m1,
-                Column('x', Integer),
-                Column('y', Integer),
-                UniqueConstraint('x')
-            )
-        Table('col_change', m2,
-                Column('x', Integer),
-                Column('y', Integer),
-                UniqueConstraint('x', 'y')
-            )
-
-        diffs = self._fixture(m1, m2)
-
-        diffs = set((cmd,
-                    ('x' in obj.name) if obj.name is not None else False)
-                    for cmd, obj in diffs)
-        if self.reports_unnamed_constraints:
-            assert ("remove_constraint", True) in diffs
-            assert ("add_constraint", False) in diffs
-
-
-
-    def test_remove_named_unique_index(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table('remove_idx', m1,
-                Column('x', Integer),
-                Index('xidx', 'x', unique=True)
-            )
-        Table('remove_idx', m2,
-                Column('x', Integer),
-            )
-
-        diffs = self._fixture(m1, m2)
-
-        diffs = set((cmd, obj.name) for cmd, obj in diffs)
-        assert ("remove_index", "xidx") in diffs
-
-
-    def test_remove_named_unique_constraint(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table('remove_idx', m1,
-                Column('x', Integer),
-                UniqueConstraint('x', name='xidx')
-            )
-        Table('remove_idx', m2,
-                Column('x', Integer),
-            )
-
-        diffs = self._fixture(m1, m2)
-
-        diffs = ((cmd, obj.name) for cmd, obj in diffs)
-        assert ("remove_constraint", "xidx") in diffs
-
-    def test_dont_add_uq_on_table_create(self):
-        m1 = MetaData()
-        m2 = MetaData()
-        Table('no_uq', m2, Column('x', String(50), unique=True))
-        diffs = self._fixture(m1, m2)
-
-        eq_(diffs[0][0], "add_table")
-        eq_(len(diffs), 1)
-        assert UniqueConstraint in set(type(c) for c in diffs[0][1].constraints)
-
-    def test_add_uq_ix_on_table_create(self):
-        m1 = MetaData()
-        m2 = MetaData()
-        Table('add_ix', m2, Column('x', String(50), unique=True, index=True))
-        diffs = self._fixture(m1, m2)
-
-        eq_(diffs[0][0], "add_table")
-        eq_(len(diffs), 2)
-        assert UniqueConstraint not in set(type(c) for c in diffs[0][1].constraints)
-        eq_(diffs[1][0], "add_index")
-        eq_(diffs[1][1].unique, True)
-
-    def test_add_ix_on_table_create(self):
-        m1 = MetaData()
-        m2 = MetaData()
-        Table('add_ix', m2, Column('x', String(50), index=True))
-        diffs = self._fixture(m1, m2)
-
-        eq_(diffs[0][0], "add_table")
-        eq_(len(diffs), 2)
-        assert UniqueConstraint not in set(type(c) for c in diffs[0][1].constraints)
-        eq_(diffs[1][0], "add_index")
-        eq_(diffs[1][1].unique, False)
-
-    def _fixture(self, m1, m2):
-        self.metadata, model_metadata = m1, m2
-        self.metadata.create_all(self.bind)
-
-        with self.bind.connect() as conn:
-            self.context = context = MigrationContext.configure(
-                connection=conn,
-                opts={
-                    'compare_type': True,
-                    'compare_server_default': True,
-                    'target_metadata': model_metadata,
-                    'upgrade_token': "upgrades",
-                    'downgrade_token': "downgrades",
-                    'alembic_module_prefix': 'op.',
-                    'sqlalchemy_module_prefix': 'sa.',
-                }
-            )
-
-            connection = context.bind
-            autogen_context = {
-                'imports': set(),
-                'connection': connection,
-                'dialect': connection.dialect,
-                'context': context
-                }
-            diffs = []
-            autogenerate._produce_net_changes(connection, model_metadata, diffs,
-                                              autogen_context,
-                                              object_filters=_default_object_filters,
-                                        )
-            return diffs
-
-    reports_unnamed_constraints = False
-
-    def setUp(self):
-        staging_env()
-        self.bind = self._get_bind()
-
-    def tearDown(self):
-        self.metadata.drop_all(self.bind)
-        clear_staging_env()
-
-    @classmethod
-    def _get_bind(cls):
-        return sqlite_db()
-
-
-
-class PGUniqueIndexTest(AutogenerateUniqueIndexTest):
-    reports_unnamed_constraints = True
-
-    @classmethod
-    def _get_bind(cls):
-        return db_for_dialect('postgresql')
-
-
-class MySQLUniqueIndexTest(AutogenerateUniqueIndexTest):
-    reports_unnamed_constraints = True
-
-    @classmethod
-    def _get_bind(cls):
-        return db_for_dialect('mysql')
 
 
 class AutogenerateCustomCompareTypeTest(AutogenTest, TestCase):
@@ -1150,3 +908,156 @@ class AutogenerateDiffOrderTest(AutogenTest, TestCase):
         eq_(diffs[1][0], 'add_table')
         eq_(diffs[1][1].name, "child")
 
+
+class CompareMetadataTest(ModelOne, AutogenTest, TestCase):
+    def test_compare_metadata(self):
+        metadata = self.m2
+
+        diffs = autogenerate.compare_metadata(self.context, metadata)
+
+        eq_(
+            diffs[0],
+            ('add_table', metadata.tables['item'])
+        )
+
+        eq_(diffs[1][0], 'remove_table')
+        eq_(diffs[1][1].name, "extra")
+
+        eq_(diffs[2][0], "add_column")
+        eq_(diffs[2][1], None)
+        eq_(diffs[2][2], "address")
+        eq_(diffs[2][3], metadata.tables['address'].c.street)
+
+        eq_(diffs[3][0], "add_column")
+        eq_(diffs[3][1], None)
+        eq_(diffs[3][2], "order")
+        eq_(diffs[3][3], metadata.tables['order'].c.user_id)
+
+        eq_(diffs[4][0][0], "modify_type")
+        eq_(diffs[4][0][1], None)
+        eq_(diffs[4][0][2], "order")
+        eq_(diffs[4][0][3], "amount")
+        eq_(repr(diffs[4][0][5]), "NUMERIC(precision=8, scale=2)")
+        eq_(repr(diffs[4][0][6]), "Numeric(precision=10, scale=2)")
+
+        eq_(diffs[5][0], 'remove_column')
+        eq_(diffs[5][3].name, 'pw')
+
+        eq_(diffs[6][0][0], "modify_default")
+        eq_(diffs[6][0][1], None)
+        eq_(diffs[6][0][2], "user")
+        eq_(diffs[6][0][3], "a1")
+        eq_(diffs[6][0][6].arg, "x")
+
+        eq_(diffs[7][0][0], 'modify_nullable')
+        eq_(diffs[7][0][5], True)
+        eq_(diffs[7][0][6], False)
+
+    def test_compare_metadata_include_object(self):
+        metadata = self.m2
+
+        def include_object(obj, name, type_, reflected, compare_to):
+            if type_ == "table":
+                return name in ("extra", "order")
+            elif type_ == "column":
+                return name != "amount"
+            else:
+                return True
+
+        context = MigrationContext.configure(
+            connection=self.bind.connect(),
+            opts={
+                'compare_type': True,
+                'compare_server_default': True,
+                'include_object': include_object,
+            }
+        )
+
+        diffs = autogenerate.compare_metadata(context, metadata)
+
+        eq_(diffs[0][0], 'remove_table')
+        eq_(diffs[0][1].name, "extra")
+
+        eq_(diffs[1][0], "add_column")
+        eq_(diffs[1][1], None)
+        eq_(diffs[1][2], "order")
+        eq_(diffs[1][3], metadata.tables['order'].c.user_id)
+
+    def test_compare_metadata_include_symbol(self):
+        metadata = self.m2
+
+        def include_symbol(table_name, schema_name):
+            return table_name in ('extra', 'order')
+
+        context = MigrationContext.configure(
+            connection=self.bind.connect(),
+            opts={
+                'compare_type': True,
+                'compare_server_default': True,
+                'include_symbol': include_symbol,
+            }
+        )
+
+        diffs = autogenerate.compare_metadata(context, metadata)
+
+        eq_(diffs[0][0], 'remove_table')
+        eq_(diffs[0][1].name, "extra")
+
+        eq_(diffs[1][0], "add_column")
+        eq_(diffs[1][1], None)
+        eq_(diffs[1][2], "order")
+        eq_(diffs[1][3], metadata.tables['order'].c.user_id)
+
+        eq_(diffs[2][0][0], "modify_type")
+        eq_(diffs[2][0][1], None)
+        eq_(diffs[2][0][2], "order")
+        eq_(diffs[2][0][3], "amount")
+        eq_(repr(diffs[2][0][5]), "NUMERIC(precision=8, scale=2)")
+        eq_(repr(diffs[2][0][6]), "Numeric(precision=10, scale=2)")
+
+        eq_(diffs[2][1][0], 'modify_nullable')
+        eq_(diffs[2][1][2], 'order')
+        eq_(diffs[2][1][5], False)
+        eq_(diffs[2][1][6], True)
+
+class PGCompareMetaData(ModelOne, AutogenTest, TestCase):
+    schema = "test_schema"
+
+    @classmethod
+    def _get_bind(cls):
+        return db_for_dialect('postgresql')
+
+    def test_compare_metadata_schema(self):
+        metadata = self.m2
+
+        context = MigrationContext.configure(
+            connection=self.bind.connect(),
+            opts={
+                "include_schemas": True
+            }
+        )
+
+        diffs = autogenerate.compare_metadata(context, metadata)
+
+        eq_(
+            diffs[0],
+            ('add_table', metadata.tables['test_schema.item'])
+        )
+
+        eq_(diffs[1][0], 'remove_table')
+        eq_(diffs[1][1].name, "extra")
+
+        eq_(diffs[2][0], "add_column")
+        eq_(diffs[2][1], "test_schema")
+        eq_(diffs[2][2], "address")
+        eq_(diffs[2][3], metadata.tables['test_schema.address'].c.street)
+
+        eq_(diffs[3][0], "add_column")
+        eq_(diffs[3][1], "test_schema")
+        eq_(diffs[3][2], "order")
+        eq_(diffs[3][3], metadata.tables['test_schema.order'].c.user_id)
+
+        eq_(diffs[4][0][0], 'modify_nullable')
+        eq_(diffs[4][0][5], False)
+        eq_(diffs[4][0][6], True)
+
