@@ -96,6 +96,16 @@ class OpTest(TestCase):
         context.assert_contains("ALTER TABLE t1 DROP COLUMN c1")
 
 
+    def test_alter_column_drop_default(self):
+        context = op_fixture('mssql')
+        op.alter_column("t", "c", server_default=None)
+        context.assert_contains("exec('alter table t drop constraint ' + @const_name)")
+
+    def test_alter_column_dont_drop_default(self):
+        context = op_fixture('mssql')
+        op.alter_column("t", "c", server_default=False)
+        context.assert_()
+
     def test_drop_column_w_check(self):
         context = op_fixture('mssql')
         op.drop_column('t1', 'c1', mssql_drop_check=True)
@@ -103,6 +113,12 @@ class OpTest(TestCase):
         context.assert_contains("exec('alter table t1 drop constraint ' + @const_name)")
         context.assert_contains("ALTER TABLE t1 DROP COLUMN c1")
 
+    def test_drop_column_w_check_quoting(self):
+        context = op_fixture('mssql')
+        op.drop_column('table', 'column', mssql_drop_check=True)
+        context.assert_contains("exec('alter table [table] drop constraint ' + @const_name)")
+        context.assert_contains("ALTER TABLE [table] DROP COLUMN [column]")
+
     def test_alter_column_nullable_w_existing_type(self):
         context = op_fixture('mssql')
         op.alter_column("t", "c", nullable=True, existing_type=Integer)
